<?php
/**
 * Util
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * @category    Util
 * @package     PasswordGenerator
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */

/**
 * This class provides a passwort generator.
 *
 * Sometimes it's neccessary to have a strong password.
 * This class generates them for you automaticly. You can define
 * what kind of characters will be used for the generation: numbers,
 * alphabetic characters or special characters like ,.+# etc.
 *
 * @category    Util
 * @package     PasswordGenerator
 * @author	"Sven Strittmatter" <ausserirdisch@sven-space.de>
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */
class Util_PasswordGenerator {
    /**
     * To set the type to numeric characters.
     *
     * @access	public
     * @var 	integer
     */
    const NUMBER	= 1;
    /**
     * To set the type to lower case letters.
     *
     * @access	public
     * @var 	integer
     */
    const LOWER	 	= 2;
    /**
     * To set the type to upper case letters.
     *
     * @access	public
     * @var 	integer
     */
    const UPPER	 	= 4;
    /**
     * To set the type to special characters.
     *
     * @access	public
     * @var 	integer
     */
    const SPECIAL	= 8;
    /**
     * How many characters the password has.
     *
     * @access	private
     * @var 	integer
     */
    private $_size = null;
    /**
     * The type of the password. Any of the constants NUMBER, LOWER, UPPER and SPECIAL
     * or any combination of it e.g. NUMBER | LOWER etc. for alhpanumeric passwords.
     *
     * @access	private
     * @var 	integer
     */
    private $_type = null;
    /**
     * All characters used by type set to PasswordGenerator::NUMBER.
     *
     * @access 	private
     * @var 	array
     */
    private $_numbers = array (
        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'
    );
    /**
     * All characters used by type set to PasswordGenerator::LOWER.
     *
     * @access 	private
     * @var 	array
     */
    private $_lower_letters = array(
        'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k',
        'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
        'w', 'x', 'y', 'z'
    );
    /**
     * All characters used by type set to PasswordGenerator::UPPER.
     *
     * @access 	private
     * @var 	array
     */
    private $_upper_letters = array(
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K',
        'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V',
        'W', 'X', 'Y', 'Z'
    );
    /**
     * All characters used by type set to PasswordGenerator::SPECIAL.
     *
     * @access 	private
     * @var 	array
     */
    private $_speciealchars = array(
        '+', '*', '#', '-', '_', ',', '.', ';', ':', '!', '���',
        '$', '%', '&', '/', '(', ')', '=', '?', '@'
    );
    /**
     * Saves every character used while generation and counts how often
     * it was used to prevent that one character is used to often and the
     * password will become weak against bruteforcing.
     *
     * @access 	private
     * @var 	array
     */
    private $_used_chars = array();
    /**
     * If the mt_srand() has been initialized.
     *
     * @access 	private
     * @var 	bool
     */
    private $_total_usable_characters = null;

    /**
     * The constructor. Initializes the type and the size of the password and also
     * the internal PHP random function. The parameter type can be any or combination of
     * the four constants. E.g.:
     * <code>
     *	// Will generates passwords with lower case and upper case letters and 10 characters:
     *	$gen = new PasswordGEnerator(PasswordGenerator::LOWER | PasswordGenerator::LOWER, 10);
     * </code>
     *
     * @access 	public
     * @throws 	InvalidArgumentException
     * @param	integer	$type	The types of characters used (default = PasswordGenerator::LOWER.
     * @param 	integer	$size	How many characters the password has (default = 8).
     */
    public function __construct($type = 2, $size = 8) {
        if ($this->_checkInput($type, 1, 15)) {
            $this->_type = $type;
        }

        if ($this->_checkInput($size, 4)) {
            $this->_size = $size;
        }

        list($usec, $sec) = explode(' ', microtime());
        mt_srand( (10000000000 * (float)$usec) ^ (float)$sec );
    }

    /**
     * Sets the type. Any of the constants NUMERIC, ALPHA and SPECIAL
     * or any combination of it e.g. NJUMERIC + ALPHA for alhpanumeric passwords.
     *
     * @access 	public
     * @throws	InvalidArgumentException
     * @param 	integer $type
     */
    public function setType($type = 2) {
        if ($this->_checkInput($type, 1, 15)) {
            $this->_type = $type;
        }
    }

    /**
     * Returns the type
     *
     * @access 	public
     * @return 	integer
     */
    public function getType() {
        return $this->_type;
    }

    /**
     * Sets the character length of the generated password.
     * Minimum value is 4 and default is 8.
     *
     * @access 	public
     * @throws	InvalidArgumentException
     * @param 	integer $size
     */
    public function setSize($size = 8) {
        if ($this->_checkInput($size, 4)) {
            $this->_size = $size;
        }
    }

    /**
     * Returns the size.
     *
     * @access 	public
     * @return 	integer
     */
    public function getSize() {
        return $this->_size;
    }

    /**
     * Returns the array with all allowd special characters.
     *
     * @access 	public
     * @return 	array
     */
    public function getAlowedSpecialChars() {
        return $this->_speciealchars;
    }

    /**
     * Generates the password an return it as string.
     *
     * @access 	public
     * @return 	string
     */
    public function generate() {
        $last_char = "";
        $password = "";
        $generater_functions = array();
        $this->_total_usable_characters = 0;

        if ($this->_type & self::NUMBER) {
            $generater_functions[] = '_generateNumber';
            $this->_total_usable_characters += count($this->_numbers);
        }
        if ($this->_type & self::LOWER) {
            $generater_functions[] = '_generateLowerLetter';
            $this->_total_usable_characters += count($this->_lower_letters);
        }
        if ($this->_type & self::UPPER) {
            $generater_functions[] = '_generateUpperLetter';
            $this->_total_usable_characters += count($this->_upper_letters);
        }
        if ($this->_type & self::SPECIAL) {
            $generater_functions[] = '_generateSpecial';
            $this->_total_usable_characters += count($this->_speciealchars);
        }

        do {
            $method = $generater_functions[mt_rand(0, count($generater_functions)-1)];
            $chr = call_user_func(array($this, $method));

            if (array_key_exists($chr, $this->_used_chars)) {
                $this->_used_chars[$chr]++;
            } else {
                $this->_used_chars[$chr] = 1;
            }

            if ($last_char != $chr) {
                $password .= $chr;
                $last_char = $chr;
            }
        } while (strlen($password) < $this->_size);

        return $password;
    }

    /**
     * Returns randomly one of the numbers stored in the numbers array ([0-9]).
     *
     * @access 	private
     * @return 	integer
     */
    private function _generateNumber() {
        return $this->_numbers[mt_rand(0, count($this->_numbers)-1)];
    }

    /**
     * Returns randomly one of the lower case letters from the lower array ([a-z]).
     *
     * @access 	private
     * @return 	char
     */
    private function _generateLowerLetter() {
        return $this->_lower_letters[mt_rand(0, count($this->_lower_letters)-1)];
    }

    /**
     * Returns randomly one of the lower case letters from the lower array ([A-Z]).
     *
     * @access 	private
     * @return 	char
     */
    private function _generateUpperLetter() {
        return $this->_upper_letters[mt_rand(0, count($this->_upper_letters)-1)];
    }

    /**
     * Returns randomly one of the special characters defined in
     * the array $this->_specialchars.
     *
     * @access 	private
     * @return 	character
     */
    private function _generateSpecial() {
        return $this->_speciealchars[mt_rand(0, count($this->_speciealchars) - 1)];
    }

    /**
     * Checks if the input value $in is an integer and between $min amd $max.
     * If $max is set to null only the $min value is checked.
     *
     * @access 	private
     * @throws 	InvalidArgumentException
     * @param 	mixed	$in		The checked value.
     * @param 	integer	$min	The lower boundary of $in.
     * @param 	integer	$max	The upper boundary of $in (default = null).
     * @return 	bool
     */
    private function _checkInput($in, $min, $max = null) {
        if (!is_integer($in)) {
            throw new InvalidArgumentException("Only integer values are allowed!");
        } else if ($min > $in) {
                throw new InvalidArgumentException("Argument must be greater euqal than $min");
            } else if ((null != $max) &&  ($max < $in)) {
                    throw new InvalidArgumentException("Argument must be less euqal than $max");
                }

        return true;
    }
}