<?php
/**
 * Console Library
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * $Author$
 * $Revisio$
 *
 * @category    Console
 * @package     Unittests
 * @copyright   Copyright (c) 2009 Sven Strittmatter
 */

// Call Console_ApplicationTest::main() if this source file is executed directly.
if (!defined('PHPUnit_MAIN_METHOD')) {
    define('PHPUnit_MAIN_METHOD', 'Console_Application_AbstractTest::main');
}

/**
 * Test helper
 */
require_once(dirname(__FILE__) . '/../../TestHelper.php');

/**
 * Console_Application_Abstract
 */
require_once('Console/Application/Abstract.php');

class Console_Application_Concrete extends Console_Application_Abstract {
    private $callCounter = 0;

    public $setUpCalled = false;
    public $findConsoleParametersCalled = false;
    public $runCalled = false;
    public $tearDownCalled = false;
    public $onErrorCalled = false;

    public $retValFindParams = true;
    public $thowsExceptionInRun = false;

    public $sigTermCalled = false;
    public $sigIntCalled = false;
    public $sigHangUpCalled = false;
    
    public static $disablePcntl = false;
   
    protected function getCallCounter() {
        $this->callCounter++;
        return $this->callCounter;
    }

    protected function run() {
        $this->runCalled = $this->getCallCounter();

        if ($this->thowsExceptionInRun) {
            throw new Console_Exception('');
        }
    }

    protected function setUp($maxExecutionTime = 10, $memoryLimit = '32M') {
        parent::setUp($maxExecutionTime, $memoryLimit);
        $this->setUpCalled = $this->getCallCounter();
    }

    protected function findConsoleParameters() {
        parent::findConsoleParameters();
        $this->findConsoleParametersCalled = $this->getCallCounter();
        return $this->retValFindParams;
    }

    protected function tearDown() {
        parent::tearDown();
        $this->tearDownCalled = $this->getCallCounter();
    }

    protected function onError($errorData) {
        parent::onError($errorData);
        $this->onErrorCalled = $this->getCallCounter();
    }

    protected function isPcntlEnabled() {
        if (self::$disablePcntl) {
            return false;
        }

        return parent::isPcntlEnabled();
    }

    public function signalTerm($signal) {
        $this->sigTermCalled = true;

    }

    public function signalInt($signal) {
        $this->sigIntCalled = true;
    }

    public function signalHangUp($signal) {
        $this->sigHangUpCalled = true;
    }
}

/**
 * @category   Console
 * @package    Unittests
 * @subpackage Console_Application
 * @copyright  Copyright (c) 2009 Sven Strittmatter
 */
class Console_Application_AbstractTest extends PHPUnit_Framework_TestCase {
    private $applicationFixture;

    /**
     * Runs the test methods of this class.
     *
     * @return void
     */
    public static function main() {
        require_once('PHPUnit/TextUI/TestRunner.php');
        $suite  = new PHPUnit_Framework_TestSuite("Console_Application_AbstractTest");
        $result = PHPUnit_TextUI_TestRunner::run($suite);
    }

    public function setUp() {
        Console_Application_Concrete::$disablePcntl = false;
        $this->applicationFixture = new Console_Application_Concrete('Concrete');
    }

    public function tearDown() {
        $this->applicationFixture = null;
    }

    /**
     *
     */
    public function testExecutes() {
        $this->applicationFixture->execute();
        $this->assertTrue($this->applicationFixture->isExecuted());
    }

    public function testCallsSetUpHook() {
        $this->applicationFixture->execute();
        // We assume that the template method execute() calls
        // setUp() first.
        $this->assertEquals(2, $this->applicationFixture->setUpCalled);
        $this->assertTrue($this->applicationFixture->isExecuted());
    }

    public function testCallsFindConsoleParmetersHook() {
        $this->applicationFixture->execute();
        // We assume that the template method execute() calls
        // findConsoleParameters() second.
        $this->assertEquals(1, $this->applicationFixture->findConsoleParametersCalled);
        $this->assertTrue($this->applicationFixture->isExecuted());
    }

    public function testExitsIfFindConsoleParametersReturnsFalse() {
        $this->applicationFixture->retValFindParams = false;
        $this->applicationFixture->execute();
        $this->assertEquals(1, $this->applicationFixture->findConsoleParametersCalled);
        $this->assertFalse($this->applicationFixture->setUpCalled);
        $this->assertFalse($this->applicationFixture->runCalled);
        $this->assertFalse($this->applicationFixture->tearDownCalled);
        $this->assertFalse($this->applicationFixture->isExecuted());
    }

    public function testCallsRunHook() {
        $this->applicationFixture->execute();
        // We assume that the template method execute() calls
        // run() third.
        $this->assertEquals(3, $this->applicationFixture->runCalled);
        $this->assertTrue($this->applicationFixture->isExecuted());
    }

    public function testCallsTearDownHook() {
        $this->applicationFixture->execute();
        // We assume that the template method execute() calls
        // tearDown() fourth.
        $this->assertEquals(4, $this->applicationFixture->tearDownCalled);
        $this->assertTrue($this->applicationFixture->isExecuted());
    }

    public function testCallsOnErrorHookByExceptions() {
        $this->applicationFixture->thowsExceptionInRun = true;
        $this->applicationFixture->execute();
        $this->assertEquals(4, $this->applicationFixture->onErrorCalled);
        $this->assertFalse($this->applicationFixture->tearDownCalled);
        $this->assertTrue($this->applicationFixture->isExecuted());
    }

    public function testIsVerbose() {
        $this->markTestIncomplete();
    }

    public function testGetHelpMessage() {
        $expectedString  = "\tconcrete.sh -v : Sets the script in verbose mode." . PHP_EOL;
        $expectedString .= "\tconcrete.sh -h : prints this help message."  . PHP_EOL . PHP_EOL;
        $expectedString .= 'For bug reporting or issues contact: foo@bar.de'  . PHP_EOL;
        $usageString = $this->applicationFixture->getHelpMessage();
        $this->assertEquals($expectedString, $usageString);

        $expectedString  = "\tconcrete.sh -v : Sets the script in verbose mode." . PHP_EOL;
        $expectedString .= "\tconcrete.sh -h : prints this help message."  . PHP_EOL . PHP_EOL;
        $expectedString .= 'Written by Sven Strittmatter.' . PHP_EOL;
        $expectedString .= 'For bug reporting or issues contact: ich@weltraumschaf.de'  . PHP_EOL;
        $usageString = $this->applicationFixture->getHelpMessage('Sven Strittmatter', 'ich@weltraumschaf.de');
        $this->assertEquals($expectedString, $usageString);
    }

    public function testGetUsage() {
        $this->assertEquals('' . PHP_EOL, $this->applicationFixture->echoMsg());
        $this->assertEquals('test message' . PHP_EOL, $this->applicationFixture->echoMsg('test message'));
    }

    public function testEchoMessage() {
        /**
         * @todo should be renamed to displayMessage()
         */
        $this->markTestIncomplete();
    }

    public function testThrowExcpetionOnCantRegisterSignalHandler() {
        $this->setExpectedException('RuntimeException');
        Console_Application_Concrete::$disablePcntl = true;
        $app = new Console_Application_Concrete('Concrete');
    }

    public function testHandleSignalTerm() {
        $this->markTestIncomplete('On Mac OS the function pcntl_signal_dispatch() does not exists!');
        posix_kill(posix_getpid(), SIGTERM);
        pcntl_signal_dispatch();
        $this->assertTrue($this->applicationFixture->sigTermCalled);
    }

    public function testHandleSignalInt() {
        $this->markTestIncomplete('On Mac OS the function pcntl_signal_dispatch() does not exists!');
        posix_kill(posix_getpid(), SIGINT);
        pcntl_signal_dispatch();
        $this->assertTrue($this->applicationFixture->sigIntCalled);
    }

    public function testHandleSignalHup() {
        $this->markTestIncomplete('On Mac OS the function pcntl_signal_dispatch() does not exists!');
        posix_kill(posix_getpid(), SIGHUP);
        pcntl_signal_dispatch();
        $this->assertTrue($this->applicationFixture->sigHangUpCalled);
    }

    public function testSetPhpEnviroment() {
        $this->markTestIncomplete();
    }

    public function testRestorePhpEnviroment() {
        $this->markTestIncomplete();
    }
}

// Call Console_ApplicationTest::main() if this source file is executed directly.
if (PHPUnit_MAIN_METHOD === 'Console_Application_AbstractTest::main') {
    Console_Application_AbstractTest::main();
}
